from Lexer import Lexer
from Parser import Parser

# Hlavná trieda interpretera, ktorá spája lexer a parser
class Calculator:
    def evaluate(self, expression):
        """
        Vyhodnotí aritmetický výraz a vráti výsledok.
        
        Args:
            expression (str): Reťazec obsahujúci aritmetický výraz
            
        Returns:
            int: Výsledok výrazu, alebo chybovú správu
        """
        try:
            lexer = Lexer(expression)   # Vytvoríme lexikálny analyzátor pre daný výraz
            parser = Parser(lexer)      # Vytvoríme syntaktický analyzátor
            result = parser.parse()     # Vyhodnotíme výraz podľa gramatiky
            return result
        except Exception as e:
            return f"Chyba: {str(e)}"   # Vrátenie chyby

def test_calculator():
    calc = Calculator()
    
    # Základné operácie
    print("2 + 3 =", calc.evaluate("2 + 3"))                 # Očakávaný výsledok: 5
    print("5 - 3 =", calc.evaluate("5 - 3"))                 # Očakávaný výsledok: 2
    print("2 * 3 =", calc.evaluate("2 * 3"))                 # Očakávaný výsledok: 6
    print("2 ^ 3 =", calc.evaluate("2 ^ 3"))                 # Očakávaný výsledok: 8
    
    # Priorita operátorov
    print("2 + 3 * 4 =", calc.evaluate("2 + 3 * 4"))         # Očakávaný výsledok: 14 (nie 20)
    print("(2 + 3) * 4 =", calc.evaluate("(2 + 3) * 4"))     # Očakávaný výsledok: 20
    print("2 * 3 ^ 2 =", calc.evaluate("2 * 3 ^ 2"))         # Očakávaný výsledok: 18 (nie 36)
    
    # Asociatívnosť
    print("5 - 3 - 1 =", calc.evaluate("5 - 3 - 1"))         # Očakávaný výsledok: 1 (ľavá asociatívnosť)
    print("2 ^ 2 ^ 3 =", calc.evaluate("2 ^ 2 ^ 3"))         # Očakávaný výsledok: 256 (pravá asociatívnosť)
    
    # Komplexný výraz
    print("2 * (3 + 4) ^ 2 - 5 =", calc.evaluate("2 * (3 + 4) ^ 2 - 5"))  # Očakávaný výsledok: 93

if __name__ == "__main__":
    calc = Calculator()
    input_text=input("> ")

    while input_text!="quite":
        if input_text != "":
            print(calc.evaluate(input_text))

        input_text=input("> ")